# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: wrapper.j2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import HeaderStr
from accelbyte_py_sdk.core import get_namespace as get_services_namespace
from accelbyte_py_sdk.core import run_request
from accelbyte_py_sdk.core import run_request_async
from accelbyte_py_sdk.core import same_doc_as

from ..models import BulkOperationResult
from ..models import CampaignCreate
from ..models import CampaignDynamicInfo
from ..models import CampaignInfo
from ..models import CampaignPagingSlicedResult
from ..models import CampaignUpdate
from ..models import CodeCreate
from ..models import CodeCreateResult
from ..models import CodeInfo
from ..models import CodeInfoPagingSlicedResult
from ..models import ErrorEntity
from ..models import RedeemHistoryPagingSlicedResult
from ..models import RedeemRequest
from ..models import RedeemResult
from ..models import ValidationErrorEntity

from ..operations.campaign import ApplyUserRedemption
from ..operations.campaign import BulkDisableCodes
from ..operations.campaign import BulkEnableCodes
from ..operations.campaign import CreateCampaign
from ..operations.campaign import CreateCodes
from ..operations.campaign import DisableCode
from ..operations.campaign import Download
from ..operations.campaign import EnableCode
from ..operations.campaign import GetCampaign
from ..operations.campaign import GetCampaignDynamic
from ..operations.campaign import GetCode
from ..operations.campaign import QueryCampaigns
from ..operations.campaign import QueryCodes
from ..operations.campaign import QueryRedeemHistory
from ..operations.campaign import UpdateCampaign
from ..models import (
    CampaignCreateRedeemTypeEnum,
    CampaignCreateStatusEnum,
    CampaignCreateTypeEnum,
)
from ..models import (
    CampaignInfoRedeemTypeEnum,
    CampaignInfoStatusEnum,
    CampaignInfoTypeEnum,
)
from ..models import CampaignUpdateRedeemTypeEnum, CampaignUpdateStatusEnum
from ..models import CodeInfoRedeemTypeEnum, CodeInfoStatusEnum, CodeInfoTypeEnum


@same_doc_as(ApplyUserRedemption)
def apply_user_redemption(
    user_id: str,
    body: Optional[RedeemRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Redeem code (applyUserRedemption)

    [SERVICE COMMUNICATION ONLY] Redeem code. If the campaign which the code belongs to is INACTIVE, the code couldn't be redeemed even if its status is ACTIVE.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:USER:{userId}:REDEMPTION", action=1 (CREATE)
      *  Returns : Redeem result

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:USER:{userId}:REDEMPTION [CREATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/redemption

        method: POST

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL RedeemRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - RedeemResult (successful operation)

        404: Not Found - ErrorEntity (37142: Code [{code}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (37172: Campaign [{campaignId}] is inactive in namespace [{namespace}] | 37173: Code [{code}] is inactive in namespace [{namespace}] | 37174: Exceeded max redeem count per code [{maxCount}] | 37175: Exceeded max redeem count per code per user [{maxCount}] | 37177: Code redemption not started | 37178: Code redemption already ended)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ApplyUserRedemption.create(
        user_id=user_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ApplyUserRedemption)
async def apply_user_redemption_async(
    user_id: str,
    body: Optional[RedeemRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Redeem code (applyUserRedemption)

    [SERVICE COMMUNICATION ONLY] Redeem code. If the campaign which the code belongs to is INACTIVE, the code couldn't be redeemed even if its status is ACTIVE.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:USER:{userId}:REDEMPTION", action=1 (CREATE)
      *  Returns : Redeem result

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:USER:{userId}:REDEMPTION [CREATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/redemption

        method: POST

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL RedeemRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - RedeemResult (successful operation)

        404: Not Found - ErrorEntity (37142: Code [{code}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (37172: Campaign [{campaignId}] is inactive in namespace [{namespace}] | 37173: Code [{code}] is inactive in namespace [{namespace}] | 37174: Exceeded max redeem count per code [{maxCount}] | 37175: Exceeded max redeem count per code per user [{maxCount}] | 37177: Code redemption not started | 37178: Code redemption already ended)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ApplyUserRedemption.create(
        user_id=user_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(BulkDisableCodes)
def bulk_disable_codes(
    campaign_id: str,
    batch_no: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk disable codes (bulkDisableCodes)

    Bulk disable codes.

    Bulk disable campaign codes, all matched codes will be disabled except those have already been redeemed.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=4 (UPDATE)
      *  Returns : the number of code actually disabled

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}/disable/bulk

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        batch_no: (batchNo) OPTIONAL int in query

    Responses:
        200: OK - BulkOperationResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkDisableCodes.create(
        campaign_id=campaign_id,
        batch_no=batch_no,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(BulkDisableCodes)
async def bulk_disable_codes_async(
    campaign_id: str,
    batch_no: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk disable codes (bulkDisableCodes)

    Bulk disable codes.

    Bulk disable campaign codes, all matched codes will be disabled except those have already been redeemed.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=4 (UPDATE)
      *  Returns : the number of code actually disabled

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}/disable/bulk

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        batch_no: (batchNo) OPTIONAL int in query

    Responses:
        200: OK - BulkOperationResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkDisableCodes.create(
        campaign_id=campaign_id,
        batch_no=batch_no,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(BulkEnableCodes)
def bulk_enable_codes(
    campaign_id: str,
    batch_no: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk enable codes (bulkEnableCodes)

    Bulk enable campaign codes.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=4 (UPDATE)
      *  Returns : the number of code actually enabled

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}/enable/bulk

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        batch_no: (batchNo) OPTIONAL int in query

    Responses:
        200: OK - BulkOperationResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkEnableCodes.create(
        campaign_id=campaign_id,
        batch_no=batch_no,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(BulkEnableCodes)
async def bulk_enable_codes_async(
    campaign_id: str,
    batch_no: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk enable codes (bulkEnableCodes)

    Bulk enable campaign codes.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=4 (UPDATE)
      *  Returns : the number of code actually enabled

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}/enable/bulk

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        batch_no: (batchNo) OPTIONAL int in query

    Responses:
        200: OK - BulkOperationResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkEnableCodes.create(
        campaign_id=campaign_id,
        batch_no=batch_no,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateCampaign)
def create_campaign(
    body: Optional[CampaignCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create campaign (createCampaign)

    Create campaign.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=1 (CREATE)
      *  Returns : created campaign

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [CREATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns

        method: POST

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL CampaignCreate in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - CampaignInfo (successful operation)

        409: Conflict - ErrorEntity (37171: Campaign [{name}] already exists in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateCampaign.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateCampaign)
async def create_campaign_async(
    body: Optional[CampaignCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create campaign (createCampaign)

    Create campaign.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=1 (CREATE)
      *  Returns : created campaign

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [CREATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns

        method: POST

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL CampaignCreate in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - CampaignInfo (successful operation)

        409: Conflict - ErrorEntity (37171: Campaign [{name}] already exists in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateCampaign.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateCodes)
def create_codes(
    campaign_id: str,
    body: Optional[CodeCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create campaign codes (createCodes)

    This API is used to create campaign codes, it will increase the batch No. based on last creation.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=1 (CREATE)
      *  Returns : number of codes created

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [CREATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}

        method: POST

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: []

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL CodeCreate in body

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - CodeCreateResult (successful operation)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateCodes.create(
        campaign_id=campaign_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateCodes)
async def create_codes_async(
    campaign_id: str,
    body: Optional[CodeCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create campaign codes (createCodes)

    This API is used to create campaign codes, it will increase the batch No. based on last creation.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=1 (CREATE)
      *  Returns : number of codes created

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [CREATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}

        method: POST

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: []

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL CodeCreate in body

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - CodeCreateResult (successful operation)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateCodes.create(
        campaign_id=campaign_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DisableCode)
def disable_code(
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Disable code (disableCode)

    Disable code.

    Disable an active code, the code can't be disabled if it has already been redeemed.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=4 (UPDATE)
      *  Returns : disabled code

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/{code}/disable

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CodeInfo (successful operation)

        404: Not Found - ErrorEntity (37142: Code [{code}] does not exist in namespace [{namespace}] | 37176: Code [{code}] has been redeemed in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DisableCode.create(
        code=code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DisableCode)
async def disable_code_async(
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Disable code (disableCode)

    Disable code.

    Disable an active code, the code can't be disabled if it has already been redeemed.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=4 (UPDATE)
      *  Returns : disabled code

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/{code}/disable

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CodeInfo (successful operation)

        404: Not Found - ErrorEntity (37142: Code [{code}] does not exist in namespace [{namespace}] | 37176: Code [{code}] has been redeemed in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DisableCode.create(
        code=code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(Download)
def download(
    campaign_id: str,
    batch_no: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Download codes (download)

    Download all or a batch of campaign's codes as a csv file.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=2 (READ)
      *  Returns : codes csv file

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}/codes.csv

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["text/csv"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        batch_no: (batchNo) OPTIONAL int in query

    Responses:
        200: OK - Any (Successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = Download.create(
        campaign_id=campaign_id,
        batch_no=batch_no,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(Download)
async def download_async(
    campaign_id: str,
    batch_no: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Download codes (download)

    Download all or a batch of campaign's codes as a csv file.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=2 (READ)
      *  Returns : codes csv file

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}/codes.csv

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["text/csv"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        batch_no: (batchNo) OPTIONAL int in query

    Responses:
        200: OK - Any (Successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = Download.create(
        campaign_id=campaign_id,
        batch_no=batch_no,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(EnableCode)
def enable_code(
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Enable code (enableCode)

    Enable code.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=4 (UPDATE)
      *  Returns : enabled code

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/{code}/enable

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CodeInfo (successful operation)

        404: Not Found - ErrorEntity (37142: Code [{code}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = EnableCode.create(
        code=code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(EnableCode)
async def enable_code_async(
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Enable code (enableCode)

    Enable code.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=4 (UPDATE)
      *  Returns : enabled code

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/{code}/enable

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CodeInfo (successful operation)

        404: Not Found - ErrorEntity (37142: Code [{code}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = EnableCode.create(
        code=code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetCampaign)
def get_campaign(
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get campaign (getCampaign)

    Get campaign info.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=2 (READ)
      *  Returns : campaign info

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns/{campaignId}

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CampaignInfo (successful operation)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCampaign.create(
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetCampaign)
async def get_campaign_async(
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get campaign (getCampaign)

    Get campaign info.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=2 (READ)
      *  Returns : campaign info

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns/{campaignId}

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CampaignInfo (successful operation)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCampaign.create(
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetCampaignDynamic)
def get_campaign_dynamic(
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get campaign dynamic (getCampaignDynamic)

    Get campaign dynamic.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=2 (READ)
      *  Returns : campaign dynamic

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns/{campaignId}/dynamic

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CampaignDynamicInfo (successful operation)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCampaignDynamic.create(
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetCampaignDynamic)
async def get_campaign_dynamic_async(
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get campaign dynamic (getCampaignDynamic)

    Get campaign dynamic.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=2 (READ)
      *  Returns : campaign dynamic

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns/{campaignId}/dynamic

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CampaignDynamicInfo (successful operation)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCampaignDynamic.create(
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetCode)
def get_code(
    code: str,
    redeemable: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get code info (getCode)

    Get campaign code, it will check code whether available to redeem if redeemable true.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=2 (READ) (READ)
      *  Returns : code info

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/{code}

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        redeemable: (redeemable) OPTIONAL bool in query

    Responses:
        200: OK - CodeInfo (successful operation)

        404: Not Found - ErrorEntity (37142: Code [{code}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (37172: Campaign [{campaignId}] is inactive in namespace [{namespace}] | 37173: Code [{code}] is inactive in namespace [{namespace}] | 37174: Exceeded max redeem count per code [{maxCount}] | 37177: Code redemption not started | 37178: Code redemption already ended)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCode.create(
        code=code,
        redeemable=redeemable,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetCode)
async def get_code_async(
    code: str,
    redeemable: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get code info (getCode)

    Get campaign code, it will check code whether available to redeem if redeemable true.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=2 (READ) (READ)
      *  Returns : code info

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/{code}

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        redeemable: (redeemable) OPTIONAL bool in query

    Responses:
        200: OK - CodeInfo (successful operation)

        404: Not Found - ErrorEntity (37142: Code [{code}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (37172: Campaign [{campaignId}] is inactive in namespace [{namespace}] | 37173: Code [{code}] is inactive in namespace [{namespace}] | 37174: Exceeded max redeem count per code [{maxCount}] | 37177: Code redemption not started | 37178: Code redemption already ended)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCode.create(
        code=code,
        redeemable=redeemable,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(QueryCampaigns)
def query_campaigns(
    limit: Optional[int] = None,
    name: Optional[str] = None,
    offset: Optional[int] = None,
    tag: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query campaigns (queryCampaigns)

    Query campaigns, if name is presented, it's fuzzy match.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=2 (READ) (READ)
      *  Returns : slice of campaigns

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        name: (name) OPTIONAL str in query

        offset: (offset) OPTIONAL int in query

        tag: (tag) OPTIONAL str in query

    Responses:
        200: OK - CampaignPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryCampaigns.create(
        limit=limit,
        name=name,
        offset=offset,
        tag=tag,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QueryCampaigns)
async def query_campaigns_async(
    limit: Optional[int] = None,
    name: Optional[str] = None,
    offset: Optional[int] = None,
    tag: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query campaigns (queryCampaigns)

    Query campaigns, if name is presented, it's fuzzy match.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=2 (READ) (READ)
      *  Returns : slice of campaigns

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        name: (name) OPTIONAL str in query

        offset: (offset) OPTIONAL int in query

        tag: (tag) OPTIONAL str in query

    Responses:
        200: OK - CampaignPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryCampaigns.create(
        limit=limit,
        name=name,
        offset=offset,
        tag=tag,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(QueryCodes)
def query_codes(
    campaign_id: str,
    active_only: Optional[bool] = None,
    batch_no: Optional[int] = None,
    code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query codes (queryCodes)

    Query campaign codes.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=2 (READ) (READ)
      *  Returns : list of codes

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        active_only: (activeOnly) OPTIONAL bool in query

        batch_no: (batchNo) OPTIONAL int in query

        code: (code) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - CodeInfoPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryCodes.create(
        campaign_id=campaign_id,
        active_only=active_only,
        batch_no=batch_no,
        code=code,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QueryCodes)
async def query_codes_async(
    campaign_id: str,
    active_only: Optional[bool] = None,
    batch_no: Optional[int] = None,
    code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query codes (queryCodes)

    Query campaign codes.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=2 (READ) (READ)
      *  Returns : list of codes

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        active_only: (activeOnly) OPTIONAL bool in query

        batch_no: (batchNo) OPTIONAL int in query

        code: (code) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - CodeInfoPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryCodes.create(
        campaign_id=campaign_id,
        active_only=active_only,
        batch_no=batch_no,
        code=code,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(QueryRedeemHistory)
def query_redeem_history(
    campaign_id: str,
    code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query redeem history (queryRedeemHistory)

    Query redeem history.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=2 (READ)
      *  Returns : slice of redeem history

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}/history

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        code: (code) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        user_id: (userId) OPTIONAL str in query

    Responses:
        200: OK - RedeemHistoryPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryRedeemHistory.create(
        campaign_id=campaign_id,
        code=code,
        limit=limit,
        offset=offset,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QueryRedeemHistory)
async def query_redeem_history_async(
    campaign_id: str,
    code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query redeem history (queryRedeemHistory)

    Query redeem history.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=2 (READ)
      *  Returns : slice of redeem history

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}/history

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        code: (code) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        user_id: (userId) OPTIONAL str in query

    Responses:
        200: OK - RedeemHistoryPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryRedeemHistory.create(
        campaign_id=campaign_id,
        code=code,
        limit=limit,
        offset=offset,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateCampaign)
def update_campaign(
    campaign_id: str,
    body: Optional[CampaignUpdate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update campaign (updateCampaign)

    Update campaign.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=4 (UPDATE)
      *  Returns : updated campaign

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns/{campaignId}

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL CampaignUpdate in body

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CampaignInfo (successful operation)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (37171: Campaign [{name}] already exists in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateCampaign.create(
        campaign_id=campaign_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateCampaign)
async def update_campaign_async(
    campaign_id: str,
    body: Optional[CampaignUpdate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update campaign (updateCampaign)

    Update campaign.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:CAMPAIGN", action=4 (UPDATE)
      *  Returns : updated campaign

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:CAMPAIGN [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns/{campaignId}

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL CampaignUpdate in body

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CampaignInfo (successful operation)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (37171: Campaign [{name}] already exists in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateCampaign.create(
        campaign_id=campaign_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service (4.44.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum


class TypeEnum(StrEnum):
    IMAGE = "image"
    VIDEO = "video"


class VideoSourceEnum(StrEnum):
    GENERIC = "generic"
    VIMEO = "vimeo"
    YOUTUBE = "youtube"


class Slide(Model):
    """Slide (Slide)

    Properties:
        alt: (alt) OPTIONAL str

        preview_url: (previewUrl) OPTIONAL str

        thumbnail_url: (thumbnailUrl) OPTIONAL str

        type_: (type) OPTIONAL Union[str, TypeEnum]

        url: (url) OPTIONAL str

        video_source: (videoSource) OPTIONAL Union[str, VideoSourceEnum]
    """

    # region fields

    alt: str  # OPTIONAL
    preview_url: str  # OPTIONAL
    thumbnail_url: str  # OPTIONAL
    type_: Union[str, TypeEnum]  # OPTIONAL
    url: str  # OPTIONAL
    video_source: Union[str, VideoSourceEnum]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_alt(self, value: str) -> Slide:
        self.alt = value
        return self

    def with_preview_url(self, value: str) -> Slide:
        self.preview_url = value
        return self

    def with_thumbnail_url(self, value: str) -> Slide:
        self.thumbnail_url = value
        return self

    def with_type(self, value: Union[str, TypeEnum]) -> Slide:
        self.type_ = value
        return self

    def with_url(self, value: str) -> Slide:
        self.url = value
        return self

    def with_video_source(self, value: Union[str, VideoSourceEnum]) -> Slide:
        self.video_source = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "alt"):
            result["alt"] = str(self.alt)
        elif include_empty:
            result["alt"] = ""
        if hasattr(self, "preview_url"):
            result["previewUrl"] = str(self.preview_url)
        elif include_empty:
            result["previewUrl"] = ""
        if hasattr(self, "thumbnail_url"):
            result["thumbnailUrl"] = str(self.thumbnail_url)
        elif include_empty:
            result["thumbnailUrl"] = ""
        if hasattr(self, "type_"):
            result["type"] = str(self.type_)
        elif include_empty:
            result["type"] = Union[str, TypeEnum]()
        if hasattr(self, "url"):
            result["url"] = str(self.url)
        elif include_empty:
            result["url"] = ""
        if hasattr(self, "video_source"):
            result["videoSource"] = str(self.video_source)
        elif include_empty:
            result["videoSource"] = Union[str, VideoSourceEnum]()
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        alt: Optional[str] = None,
        preview_url: Optional[str] = None,
        thumbnail_url: Optional[str] = None,
        type_: Optional[Union[str, TypeEnum]] = None,
        url: Optional[str] = None,
        video_source: Optional[Union[str, VideoSourceEnum]] = None,
        **kwargs,
    ) -> Slide:
        instance = cls()
        if alt is not None:
            instance.alt = alt
        if preview_url is not None:
            instance.preview_url = preview_url
        if thumbnail_url is not None:
            instance.thumbnail_url = thumbnail_url
        if type_ is not None:
            instance.type_ = type_
        if url is not None:
            instance.url = url
        if video_source is not None:
            instance.video_source = video_source
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> Slide:
        instance = cls()
        if not dict_:
            return instance
        if "alt" in dict_ and dict_["alt"] is not None:
            instance.alt = str(dict_["alt"])
        elif include_empty:
            instance.alt = ""
        if "previewUrl" in dict_ and dict_["previewUrl"] is not None:
            instance.preview_url = str(dict_["previewUrl"])
        elif include_empty:
            instance.preview_url = ""
        if "thumbnailUrl" in dict_ and dict_["thumbnailUrl"] is not None:
            instance.thumbnail_url = str(dict_["thumbnailUrl"])
        elif include_empty:
            instance.thumbnail_url = ""
        if "type" in dict_ and dict_["type"] is not None:
            instance.type_ = str(dict_["type"])
        elif include_empty:
            instance.type_ = Union[str, TypeEnum]()
        if "url" in dict_ and dict_["url"] is not None:
            instance.url = str(dict_["url"])
        elif include_empty:
            instance.url = ""
        if "videoSource" in dict_ and dict_["videoSource"] is not None:
            instance.video_source = str(dict_["videoSource"])
        elif include_empty:
            instance.video_source = Union[str, VideoSourceEnum]()
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, Slide]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[Slide]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[Slide, List[Slide], Dict[Any, Slide]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "alt": "alt",
            "previewUrl": "preview_url",
            "thumbnailUrl": "thumbnail_url",
            "type": "type_",
            "url": "url",
            "videoSource": "video_source",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "alt": False,
            "previewUrl": False,
            "thumbnailUrl": False,
            "type": False,
            "url": False,
            "videoSource": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "type": ["image", "video"],
            "videoSource": ["generic", "vimeo", "youtube"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model-init.j2

"""Auto-generated package that contains models used by the AccelByte Gaming Services Platform Service."""

__version__ = "4.44.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .achievement_info import AchievementInfo
from .action import Action
from .action import StatusEnum as ActionStatusEnum
from .action_request import ActionRequest
from .additional_data import AdditionalData
from .additional_data_entitlement import AdditionalDataEntitlement
from .admin_entitlement_decrement import AdminEntitlementDecrement
from .admin_entitlement_sold_request import AdminEntitlementSoldRequest
from .admin_order_create import AdminOrderCreate
from .admin_order_create import PlatformEnum as AdminOrderCreatePlatformEnum
from .adyen_config import AdyenConfig
from .ali_pay_config import AliPayConfig
from .app_config import AppConfig
from .app_entitlement_info import AppEntitlementInfo
from .app_entitlement_info import StatusEnum as AppEntitlementInfoStatusEnum
from .app_entitlement_info import AppTypeEnum as AppEntitlementInfoAppTypeEnum
from .app_entitlement_info import OriginEnum as AppEntitlementInfoOriginEnum
from .app_entitlement_paging_sliced_result import AppEntitlementPagingSlicedResult
from .app_info import AppInfo
from .app_info import GenresEnum as AppInfoGenresEnum
from .app_info import PlatformsEnum as AppInfoPlatformsEnum
from .app_info import PlayersEnum as AppInfoPlayersEnum
from .app_info import PrimaryGenreEnum as AppInfoPrimaryGenreEnum
from .apple_iap_config_info import AppleIAPConfigInfo
from .apple_iap_config_request import AppleIAPConfigRequest
from .apple_iap_receipt import AppleIAPReceipt
from .app_localization import AppLocalization
from .app_update import AppUpdate
from .app_update import GenresEnum as AppUpdateGenresEnum
from .app_update import PlatformsEnum as AppUpdatePlatformsEnum
from .app_update import PlayersEnum as AppUpdatePlayersEnum
from .app_update import PrimaryGenreEnum as AppUpdatePrimaryGenreEnum
from .available_comparison import AvailableComparison
from .available_comparison import ComparisonEnum as AvailableComparisonComparisonEnum
from .available_predicate import AvailablePredicate
from .available_predicate import (
    PredicateTypeEnum as AvailablePredicatePredicateTypeEnum,
)
from .available_predicate import ValueTypeEnum as AvailablePredicateValueTypeEnum
from .available_price import AvailablePrice
from .base_custom_config import BaseCustomConfig
from .base_custom_config import ConnectionTypeEnum as BaseCustomConfigConnectionTypeEnum
from .base_tls_config import BaseTLSConfig
from .basic_category_info import BasicCategoryInfo
from .basic_item import BasicItem
from .basic_item import EntitlementTypeEnum as BasicItemEntitlementTypeEnum
from .basic_item import ItemTypeEnum as BasicItemItemTypeEnum
from .basic_item import StatusEnum as BasicItemStatusEnum
from .basic_item import AppTypeEnum as BasicItemAppTypeEnum
from .basic_item import SeasonTypeEnum as BasicItemSeasonTypeEnum
from .billing_account import BillingAccount
from .billing_account import PaymentProviderEnum as BillingAccountPaymentProviderEnum
from .billing_history_info import BillingHistoryInfo
from .billing_history_info import StatusEnum as BillingHistoryInfoStatusEnum
from .billing_history_paging_sliced_result import BillingHistoryPagingSlicedResult
from .box_item import BoxItem
from .bulk_credit_request import BulkCreditRequest
from .bulk_credit_result import BulkCreditResult
from .bulk_credit_result import StatusEnum as BulkCreditResultStatusEnum
from .bulk_debit_request import BulkDebitRequest
from .bulk_debit_result import BulkDebitResult
from .bulk_debit_result import StatusEnum as BulkDebitResultStatusEnum
from .bulk_entitlement_grant_request import BulkEntitlementGrantRequest
from .bulk_entitlement_grant_result import BulkEntitlementGrantResult
from .bulk_entitlement_grant_result import (
    StatusEnum as BulkEntitlementGrantResultStatusEnum,
)
from .bulk_entitlement_revoke_result import BulkEntitlementRevokeResult
from .bulk_entitlement_revoke_result import (
    StatusEnum as BulkEntitlementRevokeResultStatusEnum,
)
from .bulk_operation_result import BulkOperationResult
from .bulk_region_data_change_request import BulkRegionDataChangeRequest
from .bundled_item_info import BundledItemInfo
from .bundled_item_info import EntitlementTypeEnum as BundledItemInfoEntitlementTypeEnum
from .bundled_item_info import ItemTypeEnum as BundledItemInfoItemTypeEnum
from .bundled_item_info import StatusEnum as BundledItemInfoStatusEnum
from .bundled_item_info import AppTypeEnum as BundledItemInfoAppTypeEnum
from .bundled_item_info import SeasonTypeEnum as BundledItemInfoSeasonTypeEnum
from .campaign_create import CampaignCreate
from .campaign_create import RedeemTypeEnum as CampaignCreateRedeemTypeEnum
from .campaign_create import StatusEnum as CampaignCreateStatusEnum
from .campaign_create import TypeEnum as CampaignCreateTypeEnum
from .campaign_dynamic_info import CampaignDynamicInfo
from .campaign_info import CampaignInfo
from .campaign_info import RedeemTypeEnum as CampaignInfoRedeemTypeEnum
from .campaign_info import StatusEnum as CampaignInfoStatusEnum
from .campaign_info import TypeEnum as CampaignInfoTypeEnum
from .campaign_paging_sliced_result import CampaignPagingSlicedResult
from .campaign_update import CampaignUpdate
from .campaign_update import RedeemTypeEnum as CampaignUpdateRedeemTypeEnum
from .campaign_update import StatusEnum as CampaignUpdateStatusEnum
from .cancel_request import CancelRequest
from .catalog_change_info import CatalogChangeInfo
from .catalog_change_info import ActionEnum as CatalogChangeInfoActionEnum
from .catalog_change_info import StatusEnum as CatalogChangeInfoStatusEnum
from .catalog_change_info import ItemTypeEnum as CatalogChangeInfoItemTypeEnum
from .catalog_change_info import TypeEnum as CatalogChangeInfoTypeEnum
from .catalog_change_paging_result import CatalogChangePagingResult
from .catalog_change_statistics import CatalogChangeStatistics
from .category_create import CategoryCreate
from .category_info import CategoryInfo
from .category_update import CategoryUpdate
from .checkout_config import CheckoutConfig
from .clawback_info import ClawbackInfo
from .clawback_info import FeedbackStatusEnum as ClawbackInfoFeedbackStatusEnum
from .clawback_info import StatusEnum as ClawbackInfoStatusEnum
from .client_request_parameter import ClientRequestParameter
from .client_transaction import ClientTransaction
from .code_create import CodeCreate
from .code_create_result import CodeCreateResult
from .code_info import CodeInfo
from .code_info import RedeemTypeEnum as CodeInfoRedeemTypeEnum
from .code_info import StatusEnum as CodeInfoStatusEnum
from .code_info import TypeEnum as CodeInfoTypeEnum
from .code_info_paging_sliced_result import CodeInfoPagingSlicedResult
from .condition_group import ConditionGroup
from .condition_group import OperatorEnum as ConditionGroupOperatorEnum
from .condition_group_validate_result import ConditionGroupValidateResult
from .condition_match_result import ConditionMatchResult
from .consumable_entitlement_revocation_config import (
    ConsumableEntitlementRevocationConfig,
)
from .consumable_entitlement_revocation_config import (
    StrategyEnum as ConsumableEntitlementRevocationConfigStrategyEnum,
)
from .consume_item import ConsumeItem
from .consume_item import ItemIdentityTypeEnum as ConsumeItemItemIdentityTypeEnum
from .credit_payload import CreditPayload
from .credit_payload import BalanceOriginEnum as CreditPayloadBalanceOriginEnum
from .credit_request import CreditRequest
from .credit_request import OriginEnum as CreditRequestOriginEnum
from .credit_request import SourceEnum as CreditRequestSourceEnum
from .credit_result import CreditResult
from .credit_revocation import CreditRevocation
from .credit_revocation import StatusEnum as CreditRevocationStatusEnum
from .credit_summary import CreditSummary
from .currency_config import CurrencyConfig
from .currency_create import CurrencyCreate
from .currency_create import CurrencyTypeEnum as CurrencyCreateCurrencyTypeEnum
from .currency_info import CurrencyInfo
from .currency_info import CurrencyTypeEnum as CurrencyInfoCurrencyTypeEnum
from .currency_summary import CurrencySummary
from .currency_summary import CurrencyTypeEnum as CurrencySummaryCurrencyTypeEnum
from .currency_update import CurrencyUpdate
from .currency_wallet import CurrencyWallet
from .customization import Customization
from .debit_by_currency_code_request import DebitByCurrencyCodeRequest
from .debit_by_currency_code_request import (
    BalanceOriginEnum as DebitByCurrencyCodeRequestBalanceOriginEnum,
)
from .debit_by_currency_code_request import (
    BalanceSourceEnum as DebitByCurrencyCodeRequestBalanceSourceEnum,
)
from .debit_by_wallet_platform_request import DebitByWalletPlatformRequest
from .debit_by_wallet_platform_request import (
    DebitBalanceSourceEnum as DebitByWalletPlatformRequestDebitBalanceSourceEnum,
)
from .debit_by_wallet_platform_request import (
    WalletPlatformEnum as DebitByWalletPlatformRequestWalletPlatformEnum,
)
from .debit_payload import DebitPayload
from .debit_payload import WalletPlatformEnum as DebitPayloadWalletPlatformEnum
from .debit_request import DebitRequest
from .debit_request import BalanceSourceEnum as DebitRequestBalanceSourceEnum
from .debit_result import DebitResult
from .delete_reward_condition_request import DeleteRewardConditionRequest
from .detailed_wallet_transaction_info import DetailedWalletTransactionInfo
from .detailed_wallet_transaction_info import (
    WalletActionEnum as DetailedWalletTransactionInfoWalletActionEnum,
)
from .detailed_wallet_transaction_paging_sliced_result import (
    DetailedWalletTransactionPagingSlicedResult,
)
from .dlc_config_reward_short_info import DLCConfigRewardShortInfo
from .dlc_config_reward_short_info import (
    DlcTypeEnum as DLCConfigRewardShortInfoDlcTypeEnum,
)
from .dlc_item import DLCItem
from .dlc_item_config_info import DLCItemConfigInfo
from .dlc_item_config_update import DLCItemConfigUpdate
from .dlc_record import DLCRecord
from .dlc_record import (
    EntitlementOriginSyncStatusEnum as DLCRecordEntitlementOriginSyncStatusEnum,
)
from .dlc_record import StatusEnum as DLCRecordStatusEnum
from .durable_entitlement_revocation_config import DurableEntitlementRevocationConfig
from .durable_entitlement_revocation_config import (
    StrategyEnum as DurableEntitlementRevocationConfigStrategyEnum,
)
from .entitlement_origin_sync_result import EntitlementOriginSyncResult
from .entitlement_config_info import EntitlementConfigInfo
from .entitlement_decrement import EntitlementDecrement
from .entitlement_decrement_result import EntitlementDecrementResult
from .entitlement_decrement_result import (
    ClazzEnum as EntitlementDecrementResultClazzEnum,
)
from .entitlement_decrement_result import (
    StatusEnum as EntitlementDecrementResultStatusEnum,
)
from .entitlement_decrement_result import (
    AppTypeEnum as EntitlementDecrementResultAppTypeEnum,
)
from .entitlement_decrement_result import (
    OriginEnum as EntitlementDecrementResultOriginEnum,
)
from .entitlement_decrement_result import (
    SourceEnum as EntitlementDecrementResultSourceEnum,
)
from .entitlement_decrement_result import TypeEnum as EntitlementDecrementResultTypeEnum
from .entitlement_grant import EntitlementGrant
from .entitlement_grant import OriginEnum as EntitlementGrantOriginEnum
from .entitlement_grant import SourceEnum as EntitlementGrantSourceEnum
from .entitlement_grant_result import EntitlementGrantResult
from .entitlement_history_info import EntitlementHistoryInfo
from .entitlement_history_info import ActionEnum as EntitlementHistoryInfoActionEnum
from .entitlement_history_info import OriginEnum as EntitlementHistoryInfoOriginEnum
from .entitlement_ifc import EntitlementIfc
from .entitlement_ifc import AppTypeEnum as EntitlementIfcAppTypeEnum
from .entitlement_ifc import ClazzEnum as EntitlementIfcClazzEnum
from .entitlement_ifc import OriginEnum as EntitlementIfcOriginEnum
from .entitlement_ifc import StatusEnum as EntitlementIfcStatusEnum
from .entitlement_ifc import TypeEnum as EntitlementIfcTypeEnum
from .entitlement_info import EntitlementInfo
from .entitlement_info import ClazzEnum as EntitlementInfoClazzEnum
from .entitlement_info import StatusEnum as EntitlementInfoStatusEnum
from .entitlement_info import AppTypeEnum as EntitlementInfoAppTypeEnum
from .entitlement_info import OriginEnum as EntitlementInfoOriginEnum
from .entitlement_info import SourceEnum as EntitlementInfoSourceEnum
from .entitlement_info import TypeEnum as EntitlementInfoTypeEnum
from .entitlement_loot_box_reward import EntitlementLootBoxReward
from .entitlement_ownership import EntitlementOwnership
from .entitlement_paging_sliced_result import EntitlementPagingSlicedResult
from .entitlement_platform_config_info import EntitlementPlatformConfigInfo
from .entitlement_platform_config_update import EntitlementPlatformConfigUpdate
from .entitlement_platform_config_update import (
    AllowedPlatformOriginsEnum as EntitlementPlatformConfigUpdateAllowedPlatformOriginsEnum,
)
from .entitlement_prechek_result import EntitlementPrechekResult
from .entitlement_revocation import EntitlementRevocation
from .entitlement_revocation import StatusEnum as EntitlementRevocationStatusEnum
from .entitlement_revocation_config import EntitlementRevocationConfig
from .entitlement_revoke_result import EntitlementRevokeResult
from .entitlement_sold_request import EntitlementSoldRequest
from .entitlement_sold_result import EntitlementSoldResult
from .entitlement_summary import EntitlementSummary
from .entitlement_summary import ClazzEnum as EntitlementSummaryClazzEnum
from .entitlement_summary import TypeEnum as EntitlementSummaryTypeEnum
from .entitlement_summary import OriginEnum as EntitlementSummaryOriginEnum
from .entitlement_update import EntitlementUpdate
from .entitlement_update import OriginEnum as EntitlementUpdateOriginEnum
from .entitlement_update import StatusEnum as EntitlementUpdateStatusEnum
from .epic_games_dlc_sync_request import EpicGamesDLCSyncRequest
from .epic_games_iap_config_info import EpicGamesIAPConfigInfo
from .epic_games_iap_config_request import EpicGamesIAPConfigRequest
from .epic_games_reconcile_request import EpicGamesReconcileRequest
from .epic_games_reconcile_result import EpicGamesReconcileResult
from .epic_games_reconcile_result import (
    StatusEnum as EpicGamesReconcileResultStatusEnum,
)
from .error_entity import ErrorEntity
from .estimated_price_info import EstimatedPriceInfo
from .event_additional_data import EventAdditionalData
from .event_payload import EventPayload
from .export_store_request import ExportStoreRequest
from .extension_fulfillment_summary import ExtensionFulfillmentSummary
from .extension_fulfillment_summary import (
    ItemTypeEnum as ExtensionFulfillmentSummaryItemTypeEnum,
)
from .external_payment_order_create import ExternalPaymentOrderCreate
from .external_payment_order_create import (
    ItemTypeEnum as ExternalPaymentOrderCreateItemTypeEnum,
)
from .field_validation_error import FieldValidationError
from .fixed_period_rotation_config import FixedPeriodRotationConfig
from .fixed_period_rotation_config import (
    BackfillTypeEnum as FixedPeriodRotationConfigBackfillTypeEnum,
)
from .fixed_period_rotation_config import RuleEnum as FixedPeriodRotationConfigRuleEnum
from .fulfill_code_request import FulfillCodeRequest
from .ful_fill_item_payload import FulFillItemPayload
from .ful_fill_item_payload import (
    ItemIdentityTypeEnum as FulFillItemPayloadItemIdentityTypeEnum,
)
from .ful_fill_item_payload import (
    EntitlementOriginEnum as FulFillItemPayloadEntitlementOriginEnum,
)
from .fulfillment_error import FulfillmentError
from .fulfillment_history_info import FulfillmentHistoryInfo
from .fulfillment_history_info import StatusEnum as FulfillmentHistoryInfoStatusEnum
from .fulfillment_history_info import (
    EntitlementOriginEnum as FulfillmentHistoryInfoEntitlementOriginEnum,
)
from .fulfillment_history_paging_sliced_result import (
    FulfillmentHistoryPagingSlicedResult,
)
from .fulfillment_item import FulfillmentItem
from .fulfillment_item import ItemTypeEnum as FulfillmentItemItemTypeEnum
from .fulfillment_request import FulfillmentRequest
from .fulfillment_request import (
    EntitlementOriginEnum as FulfillmentRequestEntitlementOriginEnum,
)
from .fulfillment_request import OriginEnum as FulfillmentRequestOriginEnum
from .fulfillment_request import SourceEnum as FulfillmentRequestSourceEnum
from .fulfillment_result import FulfillmentResult
from .fulfillment_script_create import FulfillmentScriptCreate
from .fulfillment_script_info import FulfillmentScriptInfo
from .fulfillment_script_update import FulfillmentScriptUpdate
from .full_app_info import FullAppInfo
from .full_app_info import GenresEnum as FullAppInfoGenresEnum
from .full_app_info import PlatformsEnum as FullAppInfoPlatformsEnum
from .full_app_info import PlayersEnum as FullAppInfoPlayersEnum
from .full_app_info import PrimaryGenreEnum as FullAppInfoPrimaryGenreEnum
from .full_category_info import FullCategoryInfo
from .full_item_info import FullItemInfo
from .full_item_info import EntitlementTypeEnum as FullItemInfoEntitlementTypeEnum
from .full_item_info import ItemTypeEnum as FullItemInfoItemTypeEnum
from .full_item_info import StatusEnum as FullItemInfoStatusEnum
from .full_item_info import AppTypeEnum as FullItemInfoAppTypeEnum
from .full_item_info import SeasonTypeEnum as FullItemInfoSeasonTypeEnum
from .full_item_paging_result import FullItemPagingResult
from .full_item_paging_sliced_result import FullItemPagingSlicedResult
from .full_section_info import FullSectionInfo
from .full_section_info import RotationTypeEnum as FullSectionInfoRotationTypeEnum
from .full_view_info import FullViewInfo
from .google_iap_config_info import GoogleIAPConfigInfo
from .google_iap_config_request import GoogleIAPConfigRequest
from .google_iap_receipt import GoogleIAPReceipt
from .google_receipt_resolve_result import GoogleReceiptResolveResult
from .grant_subscription_days_request import GrantSubscriptionDaysRequest
from .grpc_server_info import GrpcServerInfo
from .grpc_server_info import (
    ConnectionTypeEnumEnum as GrpcServerInfoConnectionTypeEnumEnum,
)
from .hierarchical_category_info import HierarchicalCategoryInfo
from .iap_clawback_paging_sliced_result import IAPClawbackPagingSlicedResult
from .iap_consume_history_info import IAPConsumeHistoryInfo
from .iap_consume_history_info import IapTypeEnum as IAPConsumeHistoryInfoIapTypeEnum
from .iap_consume_history_info import StatusEnum as IAPConsumeHistoryInfoStatusEnum
from .iap_consume_history_paging_sliced_result import (
    IAPConsumeHistoryPagingSlicedResult,
)
from .iap_item_config_info import IAPItemConfigInfo
from .iap_item_config_update import IAPItemConfigUpdate
from .iap_item_entry import IAPItemEntry
from .iap_item_entry import ItemIdentityTypeEnum as IAPItemEntryItemIdentityTypeEnum
from .iap_item_flat_entry import IAPItemFlatEntry
from .iap_item_flat_entry import (
    ItemIdentityTypeEnum as IAPItemFlatEntryItemIdentityTypeEnum,
)
from .iap_item_flat_entry import PlatformEnum as IAPItemFlatEntryPlatformEnum
from .iap_item_mapping_info import IAPItemMappingInfo
from .iap_order_info import IAPOrderInfo
from .iap_order_info import StatusEnum as IAPOrderInfoStatusEnum
from .iap_order_info import TypeEnum as IAPOrderInfoTypeEnum
from .iap_order_paging_sliced_result import IAPOrderPagingSlicedResult
from .image import Image
from .import_error_details import ImportErrorDetails
from .import_store_app_info import ImportStoreAppInfo
from .import_store_category_info import ImportStoreCategoryInfo
from .import_store_error import ImportStoreError
from .import_store_error import TypeEnum as ImportStoreErrorTypeEnum
from .import_store_item_info import ImportStoreItemInfo
from .import_store_item_info import ItemTypeEnum as ImportStoreItemInfoItemTypeEnum
from .import_store_result import ImportStoreResult
from .import_store_section_info import ImportStoreSectionInfo
from .import_store_view_info import ImportStoreViewInfo
from .in_game_item_sync import InGameItemSync
from .invoice_currency_summary import InvoiceCurrencySummary
from .invoice_summary import InvoiceSummary
from .item_acquire_request import ItemAcquireRequest
from .item_acquire_result import ItemAcquireResult
from .item_create import ItemCreate
from .item_create import EntitlementTypeEnum as ItemCreateEntitlementTypeEnum
from .item_create import ItemTypeEnum as ItemCreateItemTypeEnum
from .item_create import StatusEnum as ItemCreateStatusEnum
from .item_create import AppTypeEnum as ItemCreateAppTypeEnum
from .item_create import SeasonTypeEnum as ItemCreateSeasonTypeEnum
from .item_dynamic_data_info import ItemDynamicDataInfo
from .item_id import ItemId
from .item_id import StatusEnum as ItemIdStatusEnum
from .item_info import ItemInfo
from .item_info import EntitlementTypeEnum as ItemInfoEntitlementTypeEnum
from .item_info import ItemTypeEnum as ItemInfoItemTypeEnum
from .item_info import StatusEnum as ItemInfoStatusEnum
from .item_info import AppTypeEnum as ItemInfoAppTypeEnum
from .item_info import SeasonTypeEnum as ItemInfoSeasonTypeEnum
from .item_naming import ItemNaming
from .item_naming import ItemTypeEnum as ItemNamingItemTypeEnum
from .item_naming import SeasonTypeEnum as ItemNamingSeasonTypeEnum
from .item_naming import StatusEnum as ItemNamingStatusEnum
from .item_paging_sliced_result import ItemPagingSlicedResult
from .item_purchase_condition_validate_request import (
    ItemPurchaseConditionValidateRequest,
)
from .item_purchase_condition_validate_result import ItemPurchaseConditionValidateResult
from .item_return_request import ItemReturnRequest
from .item_revocation import ItemRevocation
from .item_revocation import (
    EntitlementOriginEnum as ItemRevocationEntitlementOriginEnum,
)
from .item_revocation import ItemTypeEnum as ItemRevocationItemTypeEnum
from .item_revocation import StatusEnum as ItemRevocationStatusEnum
from .item_snapshot import ItemSnapshot
from .item_snapshot import EntitlementTypeEnum as ItemSnapshotEntitlementTypeEnum
from .item_snapshot import ItemTypeEnum as ItemSnapshotItemTypeEnum
from .item_snapshot import AppTypeEnum as ItemSnapshotAppTypeEnum
from .item_snapshot import SeasonTypeEnum as ItemSnapshotSeasonTypeEnum
from .item_type_config_create import ItemTypeConfigCreate
from .item_type_config_create import ItemTypeEnum as ItemTypeConfigCreateItemTypeEnum
from .item_type_config_info import ItemTypeConfigInfo
from .item_type_config_info import ItemTypeEnum as ItemTypeConfigInfoItemTypeEnum
from .item_type_config_update import ItemTypeConfigUpdate
from .item_update import ItemUpdate
from .item_update import EntitlementTypeEnum as ItemUpdateEntitlementTypeEnum
from .item_update import ItemTypeEnum as ItemUpdateItemTypeEnum
from .item_update import AppTypeEnum as ItemUpdateAppTypeEnum
from .item_update import SeasonTypeEnum as ItemUpdateSeasonTypeEnum
from .item_update import StatusEnum as ItemUpdateStatusEnum
from .key_group_create import KeyGroupCreate
from .key_group_create import StatusEnum as KeyGroupCreateStatusEnum
from .key_group_dynamic_info import KeyGroupDynamicInfo
from .key_group_info import KeyGroupInfo
from .key_group_info import StatusEnum as KeyGroupInfoStatusEnum
from .key_group_paging_sliced_result import KeyGroupPagingSlicedResult
from .key_group_update import KeyGroupUpdate
from .key_group_update import StatusEnum as KeyGroupUpdateStatusEnum
from .key_info import KeyInfo
from .key_info import StatusEnum as KeyInfoStatusEnum
from .key_paging_slice_result import KeyPagingSliceResult
from .list_view_info import ListViewInfo
from .localization import Localization
from .loot_box_config import LootBoxConfig
from .loot_box_config import RollFunctionEnum as LootBoxConfigRollFunctionEnum
from .loot_box_plugin_config_info import LootBoxPluginConfigInfo
from .loot_box_plugin_config_info import (
    ExtendTypeEnum as LootBoxPluginConfigInfoExtendTypeEnum,
)
from .loot_box_plugin_config_update import LootBoxPluginConfigUpdate
from .loot_box_plugin_config_update import (
    ExtendTypeEnum as LootBoxPluginConfigUpdateExtendTypeEnum,
)
from .loot_box_reward import LootBoxReward
from .loot_box_reward import TypeEnum as LootBoxRewardTypeEnum
from .mock_iap_receipt import MockIAPReceipt
from .mock_iap_receipt import TypeEnum as MockIAPReceiptTypeEnum
from .mock_iap_receipt import ItemIdentityTypeEnum as MockIAPReceiptItemIdentityTypeEnum
from .notification_process_result import NotificationProcessResult
from .notification_process_result import (
    StatusEnum as NotificationProcessResultStatusEnum,
)
from .oculus_iap_config_info import OculusIAPConfigInfo
from .oculus_iap_config_request import OculusIAPConfigRequest
from .oculus_reconcile_result import OculusReconcileResult
from .oculus_reconcile_result import (
    IapOrderStatusEnum as OculusReconcileResultIapOrderStatusEnum,
)
from .oculus_reconcile_result import (
    ItemIdentityTypeEnum as OculusReconcileResultItemIdentityTypeEnum,
)
from .operation import Operation
from .operation import TypeEnum as OperationTypeEnum
from .operation import StatusEnum as OperationStatusEnum
from .operation_request import OperationRequest
from .operation_request import TypeEnum as OperationRequestTypeEnum
from .option_box_config import OptionBoxConfig
from .order import Order
from .order import PaymentProviderEnum as OrderPaymentProviderEnum
from .order import StatusEnum as OrderStatusEnum
from .order_bundle_item_info import OrderBundleItemInfo
from .order_create import OrderCreate
from .order_creation_options import OrderCreationOptions
from .order_grant_info import OrderGrantInfo
from .order_history_info import OrderHistoryInfo
from .order_history_info import ActionEnum as OrderHistoryInfoActionEnum
from .order_info import OrderInfo
from .order_info import StatusEnum as OrderInfoStatusEnum
from .order_info import PaymentProviderEnum as OrderInfoPaymentProviderEnum
from .order_paging_result import OrderPagingResult
from .order_paging_sliced_result import OrderPagingSlicedResult
from .order_refund_create import OrderRefundCreate
from .order_statistics import OrderStatistics
from .order_summary import OrderSummary
from .order_sync_result import OrderSyncResult
from .order_update import OrderUpdate
from .order_update import StatusEnum as OrderUpdateStatusEnum
from .ownership import Ownership
from .ownership_token import OwnershipToken
from .paging import Paging
from .payment_account import PaymentAccount
from .payment_account import TypeEnum as PaymentAccountTypeEnum
from .payment_callback_config_info import PaymentCallbackConfigInfo
from .payment_callback_config_update import PaymentCallbackConfigUpdate
from .payment_merchant_config_info import PaymentMerchantConfigInfo
from .payment_method import PaymentMethod
from .payment_method import PaymentProviderEnum as PaymentMethodPaymentProviderEnum
from .payment_notification_info import PaymentNotificationInfo
from .payment_notification_info import (
    NotificationSourceEnum as PaymentNotificationInfoNotificationSourceEnum,
)
from .payment_notification_info import StatusEnum as PaymentNotificationInfoStatusEnum
from .payment_notification_paging_sliced_result import (
    PaymentNotificationPagingSlicedResult,
)
from .payment_order import PaymentOrder
from .payment_order import ChannelEnum as PaymentOrderChannelEnum
from .payment_order import ItemTypeEnum as PaymentOrderItemTypeEnum
from .payment_order import PaymentProviderEnum as PaymentOrderPaymentProviderEnum
from .payment_order import StatusEnum as PaymentOrderStatusEnum
from .payment_order_charge_request import PaymentOrderChargeRequest
from .payment_order_charge_request import (
    PaymentProviderEnum as PaymentOrderChargeRequestPaymentProviderEnum,
)
from .payment_order_charge_status import PaymentOrderChargeStatus
from .payment_order_charge_status import (
    StatusEnum as PaymentOrderChargeStatusStatusEnum,
)
from .payment_order_create import PaymentOrderCreate
from .payment_order_create import ItemTypeEnum as PaymentOrderCreateItemTypeEnum
from .payment_order_create_result import PaymentOrderCreateResult
from .payment_order_create_result import (
    StatusEnum as PaymentOrderCreateResultStatusEnum,
)
from .payment_order_details import PaymentOrderDetails
from .payment_order_info import PaymentOrderInfo
from .payment_order_info import ChannelEnum as PaymentOrderInfoChannelEnum
from .payment_order_info import StatusEnum as PaymentOrderInfoStatusEnum
from .payment_order_info import ItemTypeEnum as PaymentOrderInfoItemTypeEnum
from .payment_order_info import (
    PaymentProviderEnum as PaymentOrderInfoPaymentProviderEnum,
)
from .payment_order_notify_simulation import PaymentOrderNotifySimulation
from .payment_order_notify_simulation import (
    NotifyTypeEnum as PaymentOrderNotifySimulationNotifyTypeEnum,
)
from .payment_order_notify_simulation import (
    PaymentProviderEnum as PaymentOrderNotifySimulationPaymentProviderEnum,
)
from .payment_order_paging_sliced_result import PaymentOrderPagingSlicedResult
from .payment_order_paid_result import PaymentOrderPaidResult
from .payment_order_refund import PaymentOrderRefund
from .payment_order_refund_result import PaymentOrderRefundResult
from .payment_order_refund_result import (
    StatusEnum as PaymentOrderRefundResultStatusEnum,
)
from .payment_order_sync_result import PaymentOrderSyncResult
from .payment_process_result import PaymentProcessResult
from .payment_provider_config_edit import PaymentProviderConfigEdit
from .payment_provider_config_edit import (
    AggregateEnum as PaymentProviderConfigEditAggregateEnum,
)
from .payment_provider_config_edit import (
    SpecialsEnum as PaymentProviderConfigEditSpecialsEnum,
)
from .payment_provider_config_info import PaymentProviderConfigInfo
from .payment_provider_config_info import (
    AggregateEnum as PaymentProviderConfigInfoAggregateEnum,
)
from .payment_provider_config_info import (
    SpecialsEnum as PaymentProviderConfigInfoSpecialsEnum,
)
from .payment_provider_config_paging_sliced_result import (
    PaymentProviderConfigPagingSlicedResult,
)
from .payment_request import PaymentRequest
from .payment_request import WalletPlatformEnum as PaymentRequestWalletPlatformEnum
from .payment_tax_config_edit import PaymentTaxConfigEdit
from .payment_tax_config_info import PaymentTaxConfigInfo
from .payment_token import PaymentToken
from .payment_url import PaymentUrl
from .payment_url import PaymentProviderEnum as PaymentUrlPaymentProviderEnum
from .payment_url import PaymentTypeEnum as PaymentUrlPaymentTypeEnum
from .payment_url_create import PaymentUrlCreate
from .payment_url_create import (
    PaymentProviderEnum as PaymentUrlCreatePaymentProviderEnum,
)
from .pay_pal_config import PayPalConfig
from .platform_dlc_config_info import PlatformDLCConfigInfo
from .platform_dlc_config_update import PlatformDLCConfigUpdate
from .platform_dlc_entry import PlatformDlcEntry
from .platform_dlc_entry import PlatformEnum as PlatformDlcEntryPlatformEnum
from .platform_reward import PlatformReward
from .platform_reward import TypeEnum as PlatformRewardTypeEnum
from .platform_reward_currency import PlatformRewardCurrency
from .platform_reward_item import PlatformRewardItem
from .platform_subscribe_request import PlatformSubscribeRequest
from .platform_wallet import PlatformWallet
from .platform_wallet import StatusEnum as PlatformWalletStatusEnum
from .platform_wallet import WalletStatusEnum as PlatformWalletWalletStatusEnum
from .platform_wallet_config_info import PlatformWalletConfigInfo
from .platform_wallet_config_update import PlatformWalletConfigUpdate
from .platform_wallet_config_update import (
    AllowedBalanceOriginsEnum as PlatformWalletConfigUpdateAllowedBalanceOriginsEnum,
)
from .play_station_dlc_sync_multi_service_labels_request import (
    PlayStationDLCSyncMultiServiceLabelsRequest,
)
from .play_station_dlc_sync_request import PlayStationDLCSyncRequest
from .play_station_iap_config_info import PlayStationIAPConfigInfo
from .playstation_iap_config_request import PlaystationIAPConfigRequest
from .play_station_multi_service_labels_reconcile_request import (
    PlayStationMultiServiceLabelsReconcileRequest,
)
from .play_station_reconcile_request import PlayStationReconcileRequest
from .play_station_reconcile_result import PlayStationReconcileResult
from .play_station_reconcile_result import (
    StatusEnum as PlayStationReconcileResultStatusEnum,
)
from .populated_item_info import PopulatedItemInfo
from .populated_item_info import (
    EntitlementTypeEnum as PopulatedItemInfoEntitlementTypeEnum,
)
from .populated_item_info import ItemTypeEnum as PopulatedItemInfoItemTypeEnum
from .populated_item_info import StatusEnum as PopulatedItemInfoStatusEnum
from .populated_item_info import AppTypeEnum as PopulatedItemInfoAppTypeEnum
from .populated_item_info import SeasonTypeEnum as PopulatedItemInfoSeasonTypeEnum
from .pre_check_fulfillment_request import PreCheckFulfillmentRequest
from .predicate import Predicate
from .predicate import ComparisonEnum as PredicateComparisonEnum
from .predicate import PredicateTypeEnum as PredicatePredicateTypeEnum
from .predicate_validate_result import PredicateValidateResult
from .public_custom_config_info import PublicCustomConfigInfo
from .public_custom_config_info import (
    ConnectionTypeEnum as PublicCustomConfigInfoConnectionTypeEnum,
)
from .purchase_condition import PurchaseCondition
from .purchase_condition_update import PurchaseConditionUpdate
from .purchased_item_count import PurchasedItemCount
from .recurring import Recurring
from .recurring import CycleEnum as RecurringCycleEnum
from .recurring_charge_result import RecurringChargeResult
from .redeemable_item import RedeemableItem
from .redeem_history_info import RedeemHistoryInfo
from .redeem_history_paging_sliced_result import RedeemHistoryPagingSlicedResult
from .redeem_request import RedeemRequest
from .redeem_result import RedeemResult
from .region_data_change import RegionDataChange
from .region_data_change import (
    ItemIdentityTypeEnum as RegionDataChangeItemIdentityTypeEnum,
)
from .region_data_item import RegionDataItem
from .region_data_item import CurrencyTypeEnum as RegionDataItemCurrencyTypeEnum
from .region_data_item_dto import RegionDataItemDTO
from .region_data_item_dto import CurrencyTypeEnum as RegionDataItemDTOCurrencyTypeEnum
from .request_history import RequestHistory
from .request_history import StatusEnum as RequestHistoryStatusEnum
from .requirement import Requirement
from .revocation_config_info import RevocationConfigInfo
from .revocation_config_update import RevocationConfigUpdate
from .revocation_error import RevocationError
from .revocation_history_info import RevocationHistoryInfo
from .revocation_history_info import StatusEnum as RevocationHistoryInfoStatusEnum
from .revocation_history_paging_sliced_result import RevocationHistoryPagingSlicedResult
from .revocation_plugin_config_info import RevocationPluginConfigInfo
from .revocation_plugin_config_info import (
    ExtendTypeEnum as RevocationPluginConfigInfoExtendTypeEnum,
)
from .revocation_plugin_config_update import RevocationPluginConfigUpdate
from .revocation_plugin_config_update import (
    ExtendTypeEnum as RevocationPluginConfigUpdateExtendTypeEnum,
)
from .revocation_request import RevocationRequest
from .revocation_request import SourceEnum as RevocationRequestSourceEnum
from .revocation_result import RevocationResult
from .revocation_result import StatusEnum as RevocationResultStatusEnum
from .revoke_currency import RevokeCurrency
from .revoke_currency import BalanceOriginEnum as RevokeCurrencyBalanceOriginEnum
from .revoke_entitlement import RevokeEntitlement
from .revoke_entitlement_payload import RevokeEntitlementPayload
from .revoke_entry import RevokeEntry
from .revoke_entry import TypeEnum as RevokeEntryTypeEnum
from .revoke_item import RevokeItem
from .revoke_item import EntitlementOriginEnum as RevokeItemEntitlementOriginEnum
from .revoke_item import ItemIdentityTypeEnum as RevokeItemItemIdentityTypeEnum
from .revoke_item import OriginEnum as RevokeItemOriginEnum
from .revoke_item_summary import RevokeItemSummary
from .revoke_item_summary import RevokeStatusEnum as RevokeItemSummaryRevokeStatusEnum
from .revoke_result import RevokeResult
from .revoke_result import StatusEnum as RevokeResultStatusEnum
from .revoke_use_count_request import RevokeUseCountRequest
from .reward_condition import RewardCondition
from .reward_create import RewardCreate
from .reward_info import RewardInfo
from .reward_item import RewardItem
from .reward_item import IdentityTypeEnum as RewardItemIdentityTypeEnum
from .reward_migration_result import RewardMigrationResult
from .reward_migration_result import StatusEnum as RewardMigrationResultStatusEnum
from .reward_paging_sliced_result import RewardPagingSlicedResult
from .rewards_request import RewardsRequest
from .rewards_request import (
    EntitlementOriginEnum as RewardsRequestEntitlementOriginEnum,
)
from .rewards_request import OriginEnum as RewardsRequestOriginEnum
from .rewards_request import SourceEnum as RewardsRequestSourceEnum
from .reward_update import RewardUpdate
from .sale_config import SaleConfig
from .section_create import SectionCreate
from .section_create import RotationTypeEnum as SectionCreateRotationTypeEnum
from .section_info import SectionInfo
from .section_item import SectionItem
from .section_paging_sliced_result import SectionPagingSlicedResult
from .section_plugin_config_info import SectionPluginConfigInfo
from .section_plugin_config_info import (
    ExtendTypeEnum as SectionPluginConfigInfoExtendTypeEnum,
)
from .section_plugin_config_update import SectionPluginConfigUpdate
from .section_plugin_config_update import (
    ExtendTypeEnum as SectionPluginConfigUpdateExtendTypeEnum,
)
from .section_update import SectionUpdate
from .section_update import RotationTypeEnum as SectionUpdateRotationTypeEnum
from .service_plugin_config_info import ServicePluginConfigInfo
from .service_plugin_config_update import ServicePluginConfigUpdate
from .slide import Slide
from .slide import TypeEnum as SlideTypeEnum
from .slide import VideoSourceEnum as SlideVideoSourceEnum
from .stackable_entitlement_info import StackableEntitlementInfo
from .stackable_entitlement_info import ClazzEnum as StackableEntitlementInfoClazzEnum
from .stackable_entitlement_info import StatusEnum as StackableEntitlementInfoStatusEnum
from .stackable_entitlement_info import (
    AppTypeEnum as StackableEntitlementInfoAppTypeEnum,
)
from .stackable_entitlement_info import OriginEnum as StackableEntitlementInfoOriginEnum
from .stackable_entitlement_info import SourceEnum as StackableEntitlementInfoSourceEnum
from .stackable_entitlement_info import TypeEnum as StackableEntitlementInfoTypeEnum
from .steam_achievement import SteamAchievement
from .steam_achievement_update_request import SteamAchievementUpdateRequest
from .steam_dlc_sync_request import SteamDLCSyncRequest
from .steam_iap_config import SteamIAPConfig
from .steam_iap_config_info import SteamIAPConfigInfo
from .steam_iap_config_request import SteamIAPConfigRequest
from .steam_sync_request import SteamSyncRequest
from .store_backup_info import StoreBackupInfo
from .store_create import StoreCreate
from .store_info import StoreInfo
from .store_update import StoreUpdate
from .stream_event import StreamEvent
from .stream_event_body import StreamEventBody
from .stripe_config import StripeConfig
from .sub_item_available_price import SubItemAvailablePrice
from .subscribable import Subscribable
from .subscribe_request import SubscribeRequest
from .subscription_activity_info import SubscriptionActivityInfo
from .subscription_activity_info import ActionEnum as SubscriptionActivityInfoActionEnum
from .subscription_activity_info import (
    SubscribedByEnum as SubscriptionActivityInfoSubscribedByEnum,
)
from .subscription_activity_paging_sliced_result import (
    SubscriptionActivityPagingSlicedResult,
)
from .subscription_info import SubscriptionInfo
from .subscription_info import ChargeStatusEnum as SubscriptionInfoChargeStatusEnum
from .subscription_info import StatusEnum as SubscriptionInfoStatusEnum
from .subscription_info import SubscribedByEnum as SubscriptionInfoSubscribedByEnum
from .subscription_paging_sliced_result import SubscriptionPagingSlicedResult
from .subscription_summary import SubscriptionSummary
from .subscription_summary import StatusEnum as SubscriptionSummaryStatusEnum
from .subscription_summary import (
    SubscribedByEnum as SubscriptionSummarySubscribedByEnum,
)
from .tax_result import TaxResult
from .test_result import TestResult
from .ticket_acquire_request import TicketAcquireRequest
from .ticket_acquire_result import TicketAcquireResult
from .ticket_booth_id import TicketBoothID
from .ticket_booth_id import TypeEnum as TicketBoothIDTypeEnum
from .ticket_dynamic_info import TicketDynamicInfo
from .ticket_sale_decrement_request import TicketSaleDecrementRequest
from .ticket_sale_increment_request import TicketSaleIncrementRequest
from .ticket_sale_increment_result import TicketSaleIncrementResult
from .timed_ownership import TimedOwnership
from .time_limited_balance import TimeLimitedBalance
from .tls_config import TLSConfig
from .trade_action_paging_sliced_result import TradeActionPagingSlicedResult
from .trade_chain_action_history_info import TradeChainActionHistoryInfo
from .trade_chain_action_history_info import (
    StatusEnum as TradeChainActionHistoryInfoStatusEnum,
)
from .trade_chained_action_commit_request import TradeChainedActionCommitRequest
from .trade_notification import TradeNotification
from .trade_notification import (
    PaymentProviderEnum as TradeNotificationPaymentProviderEnum,
)
from .trade_notification import StatusEnum as TradeNotificationStatusEnum
from .transaction import Transaction
from .transaction import ProviderEnum as TransactionProviderEnum
from .transaction import StatusEnum as TransactionStatusEnum
from .transaction import TypeEnum as TransactionTypeEnum
from .transaction_amount_details import TransactionAmountDetails
from .twitch_iap_config_info import TwitchIAPConfigInfo
from .twitch_iap_config_request import TwitchIAPConfigRequest
from .twitch_sync_request import TwitchSyncRequest
from .twitch_sync_result import TwitchSyncResult
from .twitch_sync_result import IapOrderStatusEnum as TwitchSyncResultIapOrderStatusEnum
from .user_dlc import UserDLC
from .user_dlc import PlatformEnum as UserDLCPlatformEnum
from .user_dlc_record import UserDLCRecord
from .user_dlc_record import (
    EntitlementOriginSyncStatusEnum as UserDLCRecordEntitlementOriginSyncStatusEnum,
)
from .user_dlc_record import PlatformEnum as UserDLCRecordPlatformEnum
from .user_dlc_record import StatusEnum as UserDLCRecordStatusEnum
from .validation_error_entity import ValidationErrorEntity
from .view_create import ViewCreate
from .view_info import ViewInfo
from .view_update import ViewUpdate
from .wallet_info import WalletInfo
from .wallet_info import StatusEnum as WalletInfoStatusEnum
from .wallet_paging_sliced_result import WalletPagingSlicedResult
from .wallet_revocation_config import WalletRevocationConfig
from .wallet_revocation_config import StrategyEnum as WalletRevocationConfigStrategyEnum
from .wallet_transaction_info import WalletTransactionInfo
from .wallet_transaction_info import (
    WalletActionEnum as WalletTransactionInfoWalletActionEnum,
)
from .wallet_transaction_paging_sliced_result import WalletTransactionPagingSlicedResult
from .wx_pay_config_info import WxPayConfigInfo
from .wx_pay_config_request import WxPayConfigRequest
from .xbl_achievement_update_request import XblAchievementUpdateRequest
from .xbl_dlc_sync_request import XblDLCSyncRequest
from .xbl_iap_config_info import XblIAPConfigInfo
from .xbl_iap_config_request import XblIAPConfigRequest
from .xbl_reconcile_request import XblReconcileRequest
from .xbl_reconcile_result import XblReconcileResult
from .xbl_reconcile_result import (
    IapOrderStatusEnum as XblReconcileResultIapOrderStatusEnum,
)
from .xbl_user_achievements import XblUserAchievements
from .xbl_user_session_request import XblUserSessionRequest
from .xbox_achievement import XboxAchievement
from .xsolla_config import XsollaConfig
from .xsolla_paywall_config import XsollaPaywallConfig
from .xsolla_paywall_config import DeviceEnum as XsollaPaywallConfigDeviceEnum
from .xsolla_paywall_config import SizeEnum as XsollaPaywallConfigSizeEnum
from .xsolla_paywall_config import ThemeEnum as XsollaPaywallConfigThemeEnum
from .xsolla_paywall_config_request import XsollaPaywallConfigRequest
from .xsolla_paywall_config_request import (
    DeviceEnum as XsollaPaywallConfigRequestDeviceEnum,
)
from .xsolla_paywall_config_request import (
    SizeEnum as XsollaPaywallConfigRequestSizeEnum,
)
from .xsolla_paywall_config_request import (
    ThemeEnum as XsollaPaywallConfigRequestThemeEnum,
)



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: operation.j2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Platform Service (4.44.0)

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Operation
from accelbyte_py_sdk.core import HeaderStr
from accelbyte_py_sdk.core import HttpResponse

from ...models import ErrorEntity
from ...models import PaymentOrderRefund
from ...models import PaymentOrderRefundResult
from ...models import ValidationErrorEntity


class RefundPaymentOrderByDedicated(Operation):
    """Refund payment order by dedicated server (refundPaymentOrderByDedicated)

    [Not Supported Yet In Starter]

    This API is used to refund payment order by paymentOrderNo from non justice service. e.g. dedicated server.

      * if the status field of response json is "REFUNDED", usually wallet paid, it indicates payment order already refunded
      * if the status field of response json is "REFUNDING", usually real money paid, platform will send notification to registered notify url once refund successfully



    Path Parameter:


         Parameter     | Type   | Required | Description
        ---------------|--------|----------|-----------------------------------------
        namespace      | String | Yes      | Namespace that payment order resides in
        paymentOrderNo | String | Yes      | Payment order number



     Request Body Parameters:


         Parameter  | Type   | Required | Description
        ------------|--------|----------|--------------------
        description | String | Yes      | Refund description



     Request Body Example:


        {

                   "description": "Repeated item."

        }

    `

    #### Refund Notification:

    It will send notification to configured web hook after refund successfully, http status code should return 200 or 204 once you resolve notification successfully, otherwise payment system will retry notification in interval

     Refund notification parameter:


         Parameter | Type   | Required | Description
        -----------|--------|----------|------------------------------------------------
        payload    | String | Yes      | Refund notification payload in json string
        sign       | String | Yes      | sha1 hex signature for payload and private key



     Refund notification Example:


        {

               "payload": "{
                   "type": "payment",
                   "nonceStr": "34c1dcf3eb58455eb161465bbfc0b590",
                   "paymentOrderNo": "18081239088",
                   "namespace": "accelbyte",
                   "targetNamespace": "game1",
                   "targetUserId": "94451623768940d58416ca33ca767ec3",
                   "extOrderNo": "123456789",
                   "sku": "sku",
                   "extUserId": "678",
                   "price": 100,
                   "paymentProvider": "XSOLLA",
                   "vat": 0,
                   "salesTax": 0,
                   "paymentProviderFee": 0,
                   "paymentMethodFee": 0,
                   "currency": {
                           "currencyCode": "USD",
                           "currencySymbol": "$",
                           "currencyType": "REAL",
                           "namespace": "accelbyte",
                           "decimals": 2
                       },
                   "status": "REFUNDED",
                   "createdTime": "2018-07-28T00:39:16.274Z",
                   "chargedTime": "2018-07-28T00:39:16.274Z",
                   "refundedTime": "2018-07-28T00:39:16.274Z"
               }",

               "sign":"e31fb92516cc9faaf50ad70343e1293acec6f3d5"

        }

    `

     Refund notification payload parameter list:


         Parameter         | Type     | Required | Description
        -------------------|----------|----------|--------------------------------------------------------------------------------------
        type               | String   | Yes      | Notification type: 'payment'
        paymentOrderNo     | String   | Yes      | Payment system generated order number
        extOrderNo         | String   | No       | External order number that passed by invoker
        namespace          | String   | Yes      | Namespace that related payment order resides in
        targetNamespace    | String   | Yes      | The game namespace
        targetUserId       | String   | Yes      | The user id in game namespace
        sku                | String   | No       | Item identify, it will return if pass it when create payment
        extUserId          | String   | No       | External user id, can be character id, it will return if pass it when create payment
        price              | int      | Yes      | Price of item
        paymentProvider    | String   | Yes      | Payment provider: xsolla/alipay/wxpay/wallet
        vat                | int      | Yes      | Payment order VAT
        salesTax           | int      | Yes      | Payment order sales tax
        paymentProviderFee | int      | Yes      | Payment provider fee
        paymentMethodFee   | int      | Yes      | Payment method fee
        currency           | Map      | Yes      | Payment order currency info
        status             | String   | Yes      | Payment order status
        statusReason       | String   | No       | Payment order refund status reason
        createdTime        | Datetime | No       | The time of the order created
        chargedTime        | Datetime | No       | The time of the order charged
        refundedTime       | Datetime | No       | The time of the order refunded
        customParameters   | Map      | No       | custom parameters, will return if pass it when create payment
        nonceStr           | String   | Yes      | Random string, max length is 32,



    Currency info parameter list:


         Parameter     | Type   | Required | Description
        ---------------|--------|----------|-----------------------------
        currencyCode   | String | Yes      | Currency Code
        currencySymbol | String | Yes      | Currency Symbol
        currencyType   | String | Yes      | Currency type(REAL/VIRTUAL)
        namespace      | String | Yes      | Currency namespace
        decimals       | int    | Yes      | Currency decimals



    #### Encryption Rule:

    Concat payload json string and private key and then do sha1Hex.

    #### Other detail info:

      * Token type : client token
      *  Required permission : resource="ADMIN:NAMESPACE:{namespace}:PAYMENT", action=4 (UPDATE)
      *  cross namespace allowed

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:PAYMENT [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/payment/orders/{paymentOrderNo}/refund

        method: PUT

        tags: ["Payment(Dedicated)"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL PaymentOrderRefund in body

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

    Responses:
        200: OK - PaymentOrderRefundResult (successful operation)

        204: No Content - (Refund successfully)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)

        409: Conflict - ErrorEntity (33172: Payment order [{paymentOrderNo}] is not refundable)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """

    # region fields

    _url: str = (
        "/platform/admin/namespaces/{namespace}/payment/orders/{paymentOrderNo}/refund"
    )
    _method: str = "PUT"
    _consumes: List[str] = ["application/json"]
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"], ["BEARER_AUTH"]]
    _location_query: str = None

    body: PaymentOrderRefund  # OPTIONAL in [body]
    namespace: str  # REQUIRED in [path]
    payment_order_no: str  # REQUIRED in [path]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "body": self.get_body_params(),
            "path": self.get_path_params(),
        }

    def get_body_params(self) -> Any:
        if not hasattr(self, "body") or self.body is None:
            return None
        return self.body.to_dict()

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "namespace"):
            result["namespace"] = self.namespace
        if hasattr(self, "payment_order_no"):
            result["paymentOrderNo"] = self.payment_order_no
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_body(self, value: PaymentOrderRefund) -> RefundPaymentOrderByDedicated:
        self.body = value
        return self

    def with_namespace(self, value: str) -> RefundPaymentOrderByDedicated:
        self.namespace = value
        return self

    def with_payment_order_no(self, value: str) -> RefundPaymentOrderByDedicated:
        self.payment_order_no = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "body") and self.body:
            result["body"] = self.body.to_dict(include_empty=include_empty)
        elif include_empty:
            result["body"] = PaymentOrderRefund()
        if hasattr(self, "namespace") and self.namespace:
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "payment_order_no") and self.payment_order_no:
            result["paymentOrderNo"] = str(self.payment_order_no)
        elif include_empty:
            result["paymentOrderNo"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, PaymentOrderRefundResult],
        Union[None, ErrorEntity, HttpResponse, ValidationErrorEntity],
    ]:
        """Parse the given response.

        200: OK - PaymentOrderRefundResult (successful operation)

        204: No Content - (Refund successfully)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)

        409: Conflict - ErrorEntity (33172: Payment order [{paymentOrderNo}] is not refundable)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return PaymentOrderRefundResult.create_from_dict(content), None
        if code == 204:
            return None, None
        if code == 404:
            return None, ErrorEntity.create_from_dict(content)
        if code == 409:
            return None, ErrorEntity.create_from_dict(content)
        if code == 422:
            return None, ValidationErrorEntity.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        namespace: str,
        payment_order_no: str,
        body: Optional[PaymentOrderRefund] = None,
        **kwargs,
    ) -> RefundPaymentOrderByDedicated:
        instance = cls()
        instance.namespace = namespace
        instance.payment_order_no = payment_order_no
        if body is not None:
            instance.body = body
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> RefundPaymentOrderByDedicated:
        instance = cls()
        if "body" in dict_ and dict_["body"] is not None:
            instance.body = PaymentOrderRefund.create_from_dict(
                dict_["body"], include_empty=include_empty
            )
        elif include_empty:
            instance.body = PaymentOrderRefund()
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "paymentOrderNo" in dict_ and dict_["paymentOrderNo"] is not None:
            instance.payment_order_no = str(dict_["paymentOrderNo"])
        elif include_empty:
            instance.payment_order_no = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "body": "body",
            "namespace": "namespace",
            "paymentOrderNo": "payment_order_no",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "body": False,
            "namespace": True,
            "paymentOrderNo": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service (4.44.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model

from ..models.localization import Localization


class FullViewInfo(Model):
    """Full view info (FullViewInfo)

    Properties:
        created_at: (createdAt) REQUIRED str

        display_order: (displayOrder) REQUIRED int

        localizations: (localizations) REQUIRED Dict[str, Localization]

        name: (name) REQUIRED str

        namespace: (namespace) REQUIRED str

        updated_at: (updatedAt) REQUIRED str

        view_id: (viewId) REQUIRED str
    """

    # region fields

    created_at: str  # REQUIRED
    display_order: int  # REQUIRED
    localizations: Dict[str, Localization]  # REQUIRED
    name: str  # REQUIRED
    namespace: str  # REQUIRED
    updated_at: str  # REQUIRED
    view_id: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_created_at(self, value: str) -> FullViewInfo:
        self.created_at = value
        return self

    def with_display_order(self, value: int) -> FullViewInfo:
        self.display_order = value
        return self

    def with_localizations(self, value: Dict[str, Localization]) -> FullViewInfo:
        self.localizations = value
        return self

    def with_name(self, value: str) -> FullViewInfo:
        self.name = value
        return self

    def with_namespace(self, value: str) -> FullViewInfo:
        self.namespace = value
        return self

    def with_updated_at(self, value: str) -> FullViewInfo:
        self.updated_at = value
        return self

    def with_view_id(self, value: str) -> FullViewInfo:
        self.view_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "display_order"):
            result["displayOrder"] = int(self.display_order)
        elif include_empty:
            result["displayOrder"] = 0
        if hasattr(self, "localizations"):
            result["localizations"] = {
                str(k0): v0.to_dict(include_empty=include_empty)
                for k0, v0 in self.localizations.items()
            }
        elif include_empty:
            result["localizations"] = {}
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "view_id"):
            result["viewId"] = str(self.view_id)
        elif include_empty:
            result["viewId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_at: str,
        display_order: int,
        localizations: Dict[str, Localization],
        name: str,
        namespace: str,
        updated_at: str,
        view_id: str,
        **kwargs,
    ) -> FullViewInfo:
        instance = cls()
        instance.created_at = created_at
        instance.display_order = display_order
        instance.localizations = localizations
        instance.name = name
        instance.namespace = namespace
        instance.updated_at = updated_at
        instance.view_id = view_id
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> FullViewInfo:
        instance = cls()
        if not dict_:
            return instance
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "displayOrder" in dict_ and dict_["displayOrder"] is not None:
            instance.display_order = int(dict_["displayOrder"])
        elif include_empty:
            instance.display_order = 0
        if "localizations" in dict_ and dict_["localizations"] is not None:
            instance.localizations = {
                str(k0): Localization.create_from_dict(v0, include_empty=include_empty)
                for k0, v0 in dict_["localizations"].items()
            }
        elif include_empty:
            instance.localizations = {}
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "viewId" in dict_ and dict_["viewId"] is not None:
            instance.view_id = str(dict_["viewId"])
        elif include_empty:
            instance.view_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, FullViewInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[FullViewInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[FullViewInfo, List[FullViewInfo], Dict[Any, FullViewInfo]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "createdAt": "created_at",
            "displayOrder": "display_order",
            "localizations": "localizations",
            "name": "name",
            "namespace": "namespace",
            "updatedAt": "updated_at",
            "viewId": "view_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "createdAt": True,
            "displayOrder": True,
            "localizations": True,
            "name": True,
            "namespace": True,
            "updatedAt": True,
            "viewId": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service (4.44.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum


class RevokeStatusEnum(StrEnum):
    COMPLETED = "COMPLETED"
    SKIPPED = "SKIPPED"


class RevokeItemSummary(Model):
    """Revoke item summary (RevokeItemSummary)

    Properties:
        item_id: (itemId) OPTIONAL str

        item_sku: (itemSku) OPTIONAL str

        item_type: (itemType) OPTIONAL str

        revoke_status: (revokeStatus) OPTIONAL Union[str, RevokeStatusEnum]
    """

    # region fields

    item_id: str  # OPTIONAL
    item_sku: str  # OPTIONAL
    item_type: str  # OPTIONAL
    revoke_status: Union[str, RevokeStatusEnum]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_item_id(self, value: str) -> RevokeItemSummary:
        self.item_id = value
        return self

    def with_item_sku(self, value: str) -> RevokeItemSummary:
        self.item_sku = value
        return self

    def with_item_type(self, value: str) -> RevokeItemSummary:
        self.item_type = value
        return self

    def with_revoke_status(
        self, value: Union[str, RevokeStatusEnum]
    ) -> RevokeItemSummary:
        self.revoke_status = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "item_id"):
            result["itemId"] = str(self.item_id)
        elif include_empty:
            result["itemId"] = ""
        if hasattr(self, "item_sku"):
            result["itemSku"] = str(self.item_sku)
        elif include_empty:
            result["itemSku"] = ""
        if hasattr(self, "item_type"):
            result["itemType"] = str(self.item_type)
        elif include_empty:
            result["itemType"] = ""
        if hasattr(self, "revoke_status"):
            result["revokeStatus"] = str(self.revoke_status)
        elif include_empty:
            result["revokeStatus"] = Union[str, RevokeStatusEnum]()
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        item_id: Optional[str] = None,
        item_sku: Optional[str] = None,
        item_type: Optional[str] = None,
        revoke_status: Optional[Union[str, RevokeStatusEnum]] = None,
        **kwargs,
    ) -> RevokeItemSummary:
        instance = cls()
        if item_id is not None:
            instance.item_id = item_id
        if item_sku is not None:
            instance.item_sku = item_sku
        if item_type is not None:
            instance.item_type = item_type
        if revoke_status is not None:
            instance.revoke_status = revoke_status
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> RevokeItemSummary:
        instance = cls()
        if not dict_:
            return instance
        if "itemId" in dict_ and dict_["itemId"] is not None:
            instance.item_id = str(dict_["itemId"])
        elif include_empty:
            instance.item_id = ""
        if "itemSku" in dict_ and dict_["itemSku"] is not None:
            instance.item_sku = str(dict_["itemSku"])
        elif include_empty:
            instance.item_sku = ""
        if "itemType" in dict_ and dict_["itemType"] is not None:
            instance.item_type = str(dict_["itemType"])
        elif include_empty:
            instance.item_type = ""
        if "revokeStatus" in dict_ and dict_["revokeStatus"] is not None:
            instance.revoke_status = str(dict_["revokeStatus"])
        elif include_empty:
            instance.revoke_status = Union[str, RevokeStatusEnum]()
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, RevokeItemSummary]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[RevokeItemSummary]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        RevokeItemSummary, List[RevokeItemSummary], Dict[Any, RevokeItemSummary]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "itemId": "item_id",
            "itemSku": "item_sku",
            "itemType": "item_type",
            "revokeStatus": "revoke_status",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "itemId": False,
            "itemSku": False,
            "itemType": False,
            "revokeStatus": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "revokeStatus": ["COMPLETED", "SKIPPED"],
        }

    # endregion static methods
